@@ -1,10 +1,14 @@
 import urllib.parse
-from typing import Any, Dict, Optional
+from typing import TYPE_CHECKING
 
 from packaging.version import Version
 
-from ..errors import ConfigurationError
-from .redis import RedisStorage
+from limits.errors import ConfigurationError
+from limits.storage.redis import RedisStorage
+from limits.typing import Dict, Optional, Union
+
+if TYPE_CHECKING:
+    import redis.sentinel
 
 
 class RedisSentinelStorage(RedisStorage):
@@ -17,7 +21,7 @@ class RedisSentinelStorage(RedisStorage):
     STORAGE_SCHEME = ["redis+sentinel"]
     """The storage scheme for redis accessed via a redis sentinel installation"""
 
-    DEFAULT_OPTIONS: Dict[str, Any] = {
+    DEFAULT_OPTIONS: Dict[str, Union[float, str, bool]] = {
         "socket_timeout": 0.2,
     }
     "Default options passed to :class:`~redis.sentinel.Sentinel`"
@@ -27,10 +31,10 @@ class RedisSentinelStorage(RedisStorage):
     def __init__(
         self,
         uri: str,
-        service_name: str = None,
-        sentinel_kwargs: Optional[Dict[str, Any]] = None,
-        **options
-    ):
+        service_name: Optional[str] = None,
+        sentinel_kwargs: Optional[Dict[str, Union[float, str, bool]]] = None,
+        **options: Union[float, str, bool]
+    ) -> None:
         """
         :param uri: url of the form
          ``redis+sentinel://host:port,host:port/service_name``
@@ -44,13 +48,13 @@ class RedisSentinelStorage(RedisStorage):
          or if the redis master host cannot be pinged.
         """
 
-        super(RedisStorage, self).__init__()
+        super(RedisStorage, self).__init__(uri, **options)
 
         parsed = urllib.parse.urlparse(uri)
         sentinel_configuration = []
         sentinel_options = sentinel_kwargs.copy() if sentinel_kwargs else {}
 
-        parsed_auth = {}
+        parsed_auth: Dict[str, Union[float, str, bool]] = {}
 
         if parsed.username:
             parsed_auth["username"] = parsed.username
@@ -69,7 +73,8 @@ class RedisSentinelStorage(RedisStorage):
         if self.service_name is None:
             raise ConfigurationError("'service_name' not provided")
 
-        self.sentinel = self.dependencies["redis.sentinel"].Sentinel(
+        sentinel_dep = self.dependencies["redis.sentinel"].module
+        self.sentinel: "redis.sentinel.Sentinel" = sentinel_dep.Sentinel(
             sentinel_configuration,
             sentinel_kwargs={**parsed_auth, **sentinel_options},
             **{**self.DEFAULT_OPTIONS, **parsed_auth, **options}
